<?php

namespace Drupal\document_ocr\Plugin\document_ocr\transformer;

use Drupal\Core\Form\FormStateInterface;

/**
 * Open AI Completions integration transformer plugin.
 *
 * @DocumentOcrTransformer(
 *   id = "openai_edits",
 *   name = @Translation("OpenAI Edits"),
 *   group = @Translation("OpenAI"),
 *   description = @Translation("Given a text and an instruction, the model will return an edited version of the text."),
 *   require_credentials = true
 * )
 */
class OpenAiEdits extends OpenAiCompletions {

  /**
   * {@inheritdoc}
   */
  public function configurationForm(array $form, FormStateInterface $form_state) {
    $this->service->setCredentials($this->getCredentials());
    $configuration = $this->getConfiguration();
    $form['instruction'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Instruction'),
      '#description' => $this->t('The instruction that tells the model how to edit the prompt.'),
      '#default_value' => !empty($configuration['instruction']) ? $configuration['instruction'] : '',
      '#placeholder' => $this->t('Fix the grammar'),
      '#rows' => 3,
      '#required' => TRUE,
    ];
    $form['model'] = [
      '#type' => 'select',
      '#title' => $this->t('Model'),
      '#description' => $this->t('The model which will generate the completion. Some models are suitable for natural language tasks, others specialize in code. <a href="@link" target="_blank">Learn more</a>.', ['@link' => 'https://platform.openai.com/docs/models']),
      '#options' => $this->getModels('edit'),
      '#default_value' => !empty($configuration['model']) ? $configuration['model'] : 'text-davinci-003',
      '#required' => TRUE,
    ];
    $form['temperature'] = [
      '#type' => 'number',
      '#title' => $this->t('Temperature'),
      '#description' => $this->t('What sampling temperature to use, between 0 and 2. Higher values like 0.8 will make the output more random, while lower values like 0.2 will make it more focused and deterministic. We generally recommend altering this or top_p but not both.'),
      '#min' => 0,
      '#max' => 2,
      '#step' => 0.01,
      '#default_value' => !empty($configuration['temperature']) ? $configuration['temperature'] : 0.0,
    ];
    $form['top_p'] = [
      '#type' => 'number',
      '#title' => $this->t('Top P'),
      '#description' => $this->t('An alternative to sampling with temperature, called nucleus sampling, where the model considers the results of the tokens with top_p probability mass. So 0.1 means only the tokens comprising the top 10% probability mass are considered. We generally recommend altering this or temperature but not both'),
      '#min' => 0,
      '#max' => 1,
      '#step' => 0.01,
      '#default_value' => !empty($configuration['top_p']) ? $configuration['top_p'] : 1.0,
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function configurationValues(array &$form, FormStateInterface $form_state) {
    return [
      'instruction' => $form_state->getValue('instruction'),
      'model' => (string) $form_state->getValue('model'),
      'temperature' => (float) $form_state->getValue('temperature'),
      'top_p' => (float) $form_state->getValue('top_p'),
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function transform($value, $configuration) {
    try {
      if (!empty($value)) {
        $this->service->setCredentials($this->getCredentials());
        $configuration['input'] = $value;
        if ($response = $this->service->editsRequest($configuration)) {
          if (!empty($response->choices[0])) {
            $choice = $response->choices[0]->toArray();
            return $choice['text'];
          }
        }
      }
    }
    catch (\Exception $ex) {
      return $value;
    }
  }

}
